%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cdf2pmf_mp:
%
% Calculate the conditional probability mass function of a copula model 
% conditioned on covariates X, integrated away by a discretization. This
% uses message passing to do it (relatively) efficiently.
%
% Input:
%
% - N: size of dataset
% - X_ws: weight for quadrature (different columns correspond to different
%         correlation coefficients)
% - tables: pre-tabulated information for each clique, for each
%           observable configuration, for each data point and 
%           and for each latent variable configuration
% - cliques: vertices in each clique
% - cliques_idx: positions of variables within each clique
% - children: meta-children of each clique vertex in the junction tree
% - arg_message: scope of message from child to parent
% - mpo: order in which messages should be computed
% - dont_collapse: if true, don't average over values of X
%   
% Output:
% 
% - pmf: conditional probability mass function for each data point
%
% Created by: Ricardo Silva, London, 28/02/2012
% University College London
% Current version: 29/02/2012

function pmf = cdf2pmf_mp(N, X_ws, tables, cliques, cliques_idx, ...
                          children, arg_message, mpo, dont_collapse)

num_cliques = length(cliques);
num_quad = size(tables{1}{1}, 2);
message = cell(num_cliques, 1);
root = mpo(end);

if nargin < 9, dont_collapse = false; end

for m = mpo
        
  bin_code = 2.^((length(cliques{m}) - 1):-1:0);  
  pre_message = tables{m};
  t_config = binary_comb(length(cliques{m}));
  num_config = size(t_config, 1);
  for h = children{m}
    bin_h_code = 2.^((length(arg_message{h}) - 1):-1:0);
    for i = 1:num_config
      p_pos = cliques_idx(m, arg_message{h});
      m_config = bin_h_code * t_config(i, p_pos)' + 1;
      pre_message{i} = pre_message{i} .* message{h}{m_config};
    end
  end
    
  pass = cliques_idx(m, arg_message{m});
  sum_away = setdiff(1:length(cliques{m}), pass);
    
  sum_delta = binary_comb(length(sum_away));
  pass_delta = binary_comb(length(pass));
  num_sum = size(sum_delta, 1);
  num_pass = size(pass_delta, 1);
    
  message{m} = cell(num_pass, 1);
  for v = 1:num_pass
    message{m}{v} = zeros(N, num_quad);
  end
  sum_pos = zeros(length(cliques{m}), 1);
    
  for i = 1:num_sum      
    sum_pos(sum_away) = sum_delta(i, :);
    for j = 1:num_pass
      sum_pos(pass) = pass_delta(j, :);
      pos_all = bin_code * sum_pos + 1;
      message{m}{j} = message{m}{j} + pre_message{pos_all};
    end
  end
    
end    

if ~dont_collapse
  pmf = mean(squeeze(message{root}{1}) * X_ws, 2);
else
  pmf = squeeze(message{root}{1});  
end
